require './lib/class_extensions.rb'

KCT_DATABASE = 'web_kub_znacky_kct'
KCT_DATABASE_USER = 'tst'
KCT_SQL_DUMP_FILE = 'data/kct/data-produkcni.sql'

def connect
  ActiveRecord::Base.establish_connection(
   :adapter  => "mysql2",          :host     => "localhost",
   :username => KCT_DATABASE_USER, :password => "",
   :database => KCT_DATABASE
  )
end
def execute(sql)
  ActiveRecord::Base.connection.execute(sql)
end
def output(result)
  if result.nil?
    puts 'OK' 
  else
    puts result.fields.join("\t")
    result.each do |row|
      puts row.join("\t")
    end
  end
end
def dosql(sql)
  output(execute(sql))
end

desc "Kontrola KČT dat a jejich konverze (import do našeho formátu)"

# Data KČT obsahují tabulky imp a imp_districts která možná plně obsahují všechno info a vše ostatní
# je z nich odvozeno.

task :refresh_kct => [:backup_kct_current, :create_kct_original, :repair_kct_original, :import_guideposts, :import_trails, :import_signs, :normalize_guideposts ]  do
end

task :backup_kct_current do
  puts  "Je treba rucne zazalohovat databaze kct_trails a kct_guideposts (viz 'po preneseni je treba udelat tohle')"
  raise
end


task :create_kct_original do
  puts "V případě chyby je třeba vytvořit prázdnou databázi '#{KCT_DATABASE}' pod uživatelem '#{KCT_DATABASE_USER}' "
  
  puts `cat #{KCT_SQL_DUMP_FILE} | perl -pe 's/ENGINE=InnoDB DEFAULT CHARSET=utf8/ENGINE=Aria DEFAULT CHARSET=utf8 COLLATE=utf8mb4_czech_ci/g;   s/utf8_unicode_ci/utf8mb4_czech_ci/g; s/CHARSET=utf8/CHARSET=utf8mb4/g; s/ENGINE=MyISAM/ENGINE=Aria/g'  | mysql -u #{KCT_DATABASE_USER} #{KCT_DATABASE} `
end

task :repair_kct_original do
  connect  
  # oprava znamych chyb v imp a tzt_tims
  dosql("UPDATE imp SET cislo_tim='MI112' WHERE cislo_tim='PI112' ") # zřejmě překlep  

  # zahození historických záznamů
  dosql("DELETE FROM imp WHERE  cis_trasy='6601' AND por_cistr IN (1,2)") # ver 3 je nejspíš nejnovější
#   dosql("DELETE FROM tzt_tims WHERE id IN ()") # ve verzi 2 tras už nefiguruje, nebo figuruje pod novým kctref
 
  # u tras co mají por_cistr 1 i 2 zachovat jen verzi 2
  res = execute("SELECT cis_trasy FROM imp as orig  WHERE orig.por_cistr=1 AND EXISTS (SELECT * FROM imp as dep WHERE orig.cis_trasy=dep.cis_trasy AND dep.por_cistr = 2) GROUP BY cis_trasy")
  res.each do |cis_trasy|
    print "Deleting historic ver of #{cis_trasy.first} ... "
    dosql("DELETE FROM imp WHERE por_cistr=1 AND cis_trasy='#{cis_trasy.first}'")
  end
  
end

task :import_guideposts do
  connect
  
  ## Import from tzt_trasy
  dosql("
   CREATE OR REPLACE TABLE `kct_guideposts` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `kctref` varchar(10) DEFAULT NULL,
    `nazev` varchar(255) DEFAULT NULL,
    `nazev_normalized` varchar(255) DEFAULT NULL,
    `vyska` int(11) DEFAULT NULL,
    `error` mediumtext,
    `typ` varchar(20) DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `kctref` (`kctref`)
   ) "
  )
  dosql("
    INSERT INTO kct_guideposts (kctref, nazev, vyska, typ)
      SELECT g.cislo_tim as kctref, g.naz_tim_r as nazev, MAX(g.vyska) as vyska, IF(pomocne='F', 'běžný', 'pomocný') 
        FROM imp as g 
        WHERE odbocka='F' AND nav_txt = 'F'
        GROUP BY kctref
   "
  )
  dosql("
    INSERT INTO kct_guideposts (kctref, nazev, vyska, typ)
      (SELECT g.konc_tim as kctref, g.naz_tim_r as nazev, NULL as vyska, 'konec odbočky'
        FROM imp as g
        WHERE odbocka='T' AND g.konc_tim<>'' AND nav_txt = 'F'
        GROUP BY kctref)
      ON DUPLICATE KEY UPDATE kctref=kctref
   "
  )
  ## zkontrouj zda sedí s údaji s tzt_tims
  
  ## zkontroluj zda všechny kct_guideposts mají údaje v imp
  result = execute("SELECT kctref, nazev, vyska FROM kct_guideposts")
  result.each do |kctref, nazev, vyska|
    print "Checking #{nazev} #{kctref} (#{vyska}) ... "
    res = execute("SELECT cislo_tim, naz_tim_r, MAX(vyska) as vyska FROM imp WHERE cislo_tim='#{kctref}' AND odbocka='F'  AND nav_txt = 'F'  GROUP BY vyska ")
    if res.size > 0
      if (res.first[0..1] != [kctref, nazev]) or (res.first[2] != vyska.to_s)
        raise "Nekonzistence údajů imp: #{res.first.inspect} != kg: #{[kctref, nazev, vyska].inspect}"
      end
    else
      print  " odbočka "
      res = execute("SELECT konc_tim, naz_tim_r, NULL FROM imp WHERE konc_tim='#{kctref}' AND odbocka='T'  AND nav_txt = 'F'")
      raise "Odbočka nenalezena " if res.size < 1
      if (res.first[0..1] != [kctref, nazev]) or (res.first[2].to_i != 0)
        raise "Nekonzistence údajů #{res.first.inspect} != #{[kctref, nazev, 0].inspect}" 
      end
    end
    puts 'OK'
  end
  
  ## zkontroluj zda všechny imp mají údaje v kct_guideposts
  result = execute("SELECT cislo_tim FROM imp WHERE nav_txt = 'F' AND odbocka='F'")
  result.each do |kctref, _|
    print "Checking imp #{kctref} ... "
    res = execute("SELECT * FROM kct_guideposts WHERE kctref='#{kctref}' ")
    raise "Nekonzistence #{res.to_a.inspect}" unless res.size == 1
    puts 'OK'
  end

  except = ["CH037", "CH038", "CH039", "CH042", "CH043", "CH044", "CH045", "CH171", "CH173", "CH504", 
   "CV030", "CV071", "DC128", "DC364", "KT011", "KT175", "KT302", "KV242", "KV243", "KV244", "KV712", 
   "PJ390", "PJ517", "PJ519", "PJ523", "PJ535", "PJ536", "PJ539", "PJ541", "PJ547", "PJ548", "PJ549", "PJ608", 
   "PJ611", "PJ612", "PJ613", "PJ645", "PJ647", "PJ648", "PJ677", "PJ703", "PJ704", "PJ705", "PJ706", "PJ707",
   "PJ713", "PJ714", "PJ745", "PJ747", "PJ755", "PJ767", "PJ776", "PJ777", "PJ778", "PJ779", "PJ780", 
   "PS039", "PS040", "PS041", "PS043", "PS044", "PS100", "PS108", "PS119", "PS120", "PS121", "PS149", "PS224", 
   "PS225", "PS232", "PS266", "PS290", "RO214", "RO282", "RO703", "RO704", 
   "SE025", "SE097", "SE104", "SE406", 
   "TC039", "TC040", "TC145", "TC149", "TC151", "TC152", "TC153", "TC158", "TC181", "TC404", "TC601", "TC605"]
  # toto casem prozkoumat zda je ok
  
  
  ## zkontroluj zda všechny tzt_tims mají údaje v kct_guideposts
  result = execute("SELECT id, nazev, nadm_vyska FROM tzt_tims")
  result.each do |kctref, nazev, vyska|
    next if except.include? kctref  # TC040 je specialni
    print "Checking tzt_tims #{nazev} #{kctref} (#{vyska}) ... "
    res = execute("SELECT kctref, nazev, vyska, typ FROM kct_guideposts WHERE kctref='#{kctref}' ")
    raise "Nekonzistence #{res.to_a.inspect}" unless res.size == 1
    if (res.first[0..1] != [kctref, nazev]) 
      raise "Nekonzistence údajů kg:#{res.first[0..1]} != tzt_tims:#{[kctref, nazev]}"
    end
    if res.first[3]!='konec odbočky' and (res.first[2].to_i != vyska.to_i)
      raise "Nekonzistence výšky kg: #{res.first[2]} != tzt_tims:#{vyska}"      
    end
    puts 'OK'
  end
end

task :import_trails do
  connect
  ## Import from tzt_trasy
  dosql('
   CREATE OR REPLACE TABLE `kct_trails` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `kctref` varchar(10) DEFAULT NULL,
    `barva` varchar(20) DEFAULT NULL,
    `typ` varchar(20) DEFAULT NULL,
    `priority` int(11) DEFAULT NULL,
    `error` mediumtext,
    PRIMARY KEY (`id`),
    UNIQUE KEY `kctref` (`kctref`)
   )'
  )
  res = execute('
   INSERT INTO kct_trails (kctref, barva, typ)
    SELECT t.id as kctref, barvy.barva, typy.typ
     FROM tzt_trasy as t
     JOIN tzt_barvy_tras as barvy ON (t.barva_trasy_id = barvy.id)
     JOIN tzt_typy_tras as typy ON  (t.typ_trasy_id = typy.id)
   '
  )
  ## Zkontroluj zda to sedí s informacemi z imp
  barvy = {"CP"=>"červená", "MP"=>"modrá", "XP"=>"zelená", "YP"=>"žlutá", "CT"=>"červená", "YT"=>"žlutá", "ZZ"=>"zelená", "MT"=>"modrá", "XT"=>"zelená", "BB"=>"červená"}
  typy = {"CP"=>"pěší", "MP"=>"pěší", "XP"=>"pěší", "YP"=>"pěší", "CT"=>"místní", "YT"=>"místní", "ZZ"=>"naučná", "MT"=>"místní", "XT"=>"místní", "BB"=>"pěší"}
  druhy = {"P"=>"pěší", "N"=>"naučná", "M"=>"místní", "B"=>"pěší"}
  
  result = execute('SELECT kctref,barva,typ FROM kct_trails ORDER BY kctref')
  result.each do |item|
    kctref,barva,typ = item
    print "Checking #{kctref} #{barva}/#{typ} ... "
    res = execute("SELECT barva,druh FROM imp WHERE cis_trasy = #{kctref} GROUP BY barva,druh")
    raise "Nekonzistence barvy u #{kctref}" unless res.size == 1
    impkod,impdruh = res.first
    raise "Nekonzistence barvy" unless barvy[impkod]== barva
    raise "Nekonzistence typu" unless typy[impkod] == typ
    raise "Nekonzistence typu (druh)" unless druhy[impdruh] == typ
    puts "#{impkod}/#{impdruh} OK"
  end
  
end

task :import_signs do
  connect
  dosql('
    CREATE OR REPLACE TABLE `kct_kilometraz` (
      `id` int(11) NOT NULL AUTO_INCREMENT,
      `kct_trail_id` int(11) DEFAULT NULL,
      `kct_guidepost_id` int(11) DEFAULT NULL,
      `km` decimal(10,2) DEFAULT NULL,
      PRIMARY KEY (`id`),
      KEY `guidepost_id` (`kct_guidepost_id`),
      KEY `kct_trail` (`kct_trail_id`)
      ) 
  ')
  dosql("
    INSERT INTO kct_kilometraz (kct_trail_id, kct_guidepost_id, km)
      SELECT kct_trails.id, kct_guideposts.id, CONVERT(REPLACE(kilom_tam,',','.'), DECIMAL(10,2)) 
        FROM `imp` 
        LEFT JOIN kct_guideposts ON kct_guideposts.kctref=cislo_tim
        LEFT JOIN kct_trails ON kct_trails.kctref=cis_trasy  
        WHERE nav_txt='F' AND odbocka='F'
   "
  )
  
end

task :normalize_guideposts => [:environment] do
  puts "Normalizing names"
  KctGuidepost.all.each do |kct_guidepost|
    print "#{kct_guidepost.id},"
    kct_guidepost.nazev_normalized = kct_guidepost.nazev.normalize
    kct_guidepost.save
  end
  puts "\nOK"
end

task 'po preneseni je treba udelat tohle' do
# UPDATE web_kub_znacky.kct_trails as new
#   JOIN web_kub_znacky_bak.kct_trails as old ON old.kctref = new.kctref
#   SET new.error = old.error
#   WHERE old.error != ''
#   
# UPDATE web_kub_znacky.kct_guideposts as new
#   JOIN web_kub_znacky_bak.kct_guideposts as old ON old.kctref = new.kctref
#   SET new.error = old.error
#   WHERE old.error != ''
end

task check_duplicate_kct_signs: :environment do
  result = (execute('
    SELECT kct_trails.kctref, km, kct_guideposts.kctref, kct_guideposts.nazev, COUNT(*) as count
      FROM kct_kilometraz
      JOIN kct_guideposts ON kct_guidepost_id = kct_guideposts.id
      JOIN kct_trails ON kct_trail_id = kct_trails.id
      GROUP BY kct_trail_id, kct_guidepost_id, km
      HAVING count>1
    '
  ))
  output(result)
end


# UNUSED: texty, spoj_texty (jeden text muze byt relevantni k vice guidepostum)
